@@ -1014,7 +1014,7 @@ void registry::detach(mapped *rr)
 	locking.release();
 }
 
-unsigned registry::mapped::setTarget(Socket::address& target_addr, time_t lease, const char *target_contact)
+unsigned registry::mapped::setTarget(Socket::address& target_addr, time_t lease, const char *target_contact, const char *target_network, struct sockaddr *target_peering)
 {
 	assert(!isnull(target_addr));
 	assert(target_contact != NULL && *target_contact != 0);
@@ -1069,11 +1069,13 @@ unsigned registry::mapped::setTarget(Socket::address& target_addr, time_t lease,
 			tp->index.address = (struct sockaddr *)(&tp->address);
 			tp->index.enlist(&addresses[Socket::keyindex(tp->index.address, keysize)]);
 		}
-		stack::getInterface((struct sockaddr *)(&tp->iface), (struct sockaddr *)(&tp->address));
 		if(origin) 
 			delete origin;
 	}
+	Socket::store(&tp->peering, target_peering);
+	String::set(tp->network, sizeof(tp->network), target_network);
 	String::set(tp->contact, sizeof(tp->contact), target_contact);
+	String::set(network, sizeof(network), target_network);
 	uri::userid(target_contact, remote, sizeof(remote));
 	locking.share();
 	return 1;
@@ -1280,7 +1282,7 @@ bool registry::mapped::refresh(Socket::address& saddr, time_t lease, const char
 	return false;
 }
 
-unsigned registry::mapped::addTarget(Socket::address& target_addr, time_t lease, const char *target_contact)
+unsigned registry::mapped::addTarget(Socket::address& target_addr, time_t lease, const char *target_contact, const char *target_network, struct sockaddr *target_peering)
 {
 	assert(!isnull(target_addr));
 	assert(target_contact != NULL && *target_contact != 0);
@@ -1325,7 +1327,9 @@ unsigned registry::mapped::addTarget(Socket::address& target_addr, time_t lease,
 		if(tp->expires < now)
 			time(&tp->created);
 		tp->expires = lease;
+		Socket::store(&tp->peering, target_peering);
 		String::set(tp->contact, sizeof(tp->contact), target_contact);
+		String::set(tp->network, sizeof(tp->network), target_network);
 		locking.share();
 		return count;
 	}
@@ -1340,6 +1344,7 @@ unsigned registry::mapped::addTarget(Socket::address& target_addr, time_t lease,
 		expired->enlist(&internal.targets);
 		expired->status = registry::target::READY;
 		memcpy(&contact, oi, len);
+		String::set(network, sizeof(network), target_network);
 		uri::userid(target_contact, remote, sizeof(remote));
 		if(origin)
 			delete origin;
@@ -1348,8 +1353,9 @@ unsigned registry::mapped::addTarget(Socket::address& target_addr, time_t lease,
 	time(&expired->created);
 	expired->expires = lease;
 	memcpy(&expired->address, ai, len);
-	stack::getInterface((struct sockaddr *)(&expired->iface), (struct sockaddr *)(&expired->address));
+	Socket::store(&expired->peering, target_peering);
 	String::set(expired->contact, sizeof(expired->contact), target_contact);
+	String::set(expired->network, sizeof(expired->network), target_network);
 	expired->index.registry = this;
 	expired->index.address = (struct sockaddr *)(&expired->address);
 	expired->index.enlist(&addresses[Socket::keyindex(expired->index.address, keysize)]); 
@@ -1362,6 +1368,7 @@ unsigned registry::mapped::setTargets(Socket::address& target_addr)
 {
 	assert(!isnull(target_addr));
 
+	stack::subnet *subnet;
 	struct addrinfo *al;
 	linked_pointer<target> tp;
 	socklen_t len;
@@ -1388,11 +1395,22 @@ unsigned registry::mapped::setTargets(Socket::address& target_addr)
 
 		tp = new target;
 		time(&tp->created);
+		subnet = server::getPolicy(al->ai_addr);
+		if(subnet) {
+			tp->peering = subnet->iface;
+			String::set(tp->network, sizeof(tp->network), subnet->netname);
+		}
+		else {
+			service::published(&tp->peering);
+			String::set(tp->network, sizeof(tp->network), "*");
+		}
+		server::release(subnet);
+		String::set(network, sizeof(network), tp->network);
 		memcpy(&tp->address, al->ai_addr, len);
 		memcpy(&contact, &tp->address, len);
 		remote[0] = 0;
-		stack::getInterface((struct sockaddr *)(&tp->iface), (struct sockaddr *)(&tp->address));
 		stack::sipAddress(&tp->address, tp->contact, userid);
+
 		tp->expires = 0l;
 		tp->status = registry::target::READY;
 		tp->enlist(&internal.targets);
