@@ -15,6 +15,7 @@
 
 #include "server.h"
 #include <signal.h>
+#include <ctype.h>
 
 NAMESPACE_SIPWITCH
 using namespace UCOMMON_NAMESPACE;
@@ -199,18 +200,6 @@ bool server::publish(MappedRegistry *rr, const char *msgtype, const char *event,
 	return rtn;
 }
 
-bool server::classify(rtpproxy::session *sid, rtpproxy::session *src, struct sockaddr *addr)
-{
-	linked_pointer<modules::sipwitch> cb = getModules();
-	bool rtn = false;
-
-	while(!rtn && is(cb)) {
-		rtn = cb->classifier(sid, src, addr);
-		cb.next();
-	}
-	return rtn;
-}
-
 service::keynode *server::find(const char *id)
 {
 	assert(id != NULL && *id != 0);
@@ -356,17 +345,18 @@ void server::confirm(const char *user)
 	fsys::close(dir);
 
 	mp = (caddr_t)alloc(sizeof(cidr));
-	new(mp) cidr(&acl, "127.0.0.0/8", "loopback");
+	new(mp) stack::subnet(&acl, "127.0.0.0/8", "loopback");
 
 	mp = (caddr_t)alloc(sizeof(cidr));
-	new(mp) cidr(&acl, "::1", "loopback");
+	new(mp) stack::subnet(&acl, "::1", "loopback");
 
 	node = access->getFirst();
 	while(node) {
 		id = node->getId();
+		leaf = NULL;
 		if(id && node->getPointer()) {
 			mp = (caddr_t)alloc(sizeof(cidr));
-			new(mp) cidr(&acl, node->getPointer(), id);
+			new(mp) stack::subnet(&acl, node->getPointer(), id);
 		}
 		node.next();
 	}
@@ -447,7 +437,7 @@ void server::release(usernode& user)
 	user.heap = NULL;
 }
 
-void server::release(cidr *access)
+void server::release(stack::subnet *access)
 {
 	if(access)
 		locking.release();
@@ -467,18 +457,18 @@ bool server::isLocal(struct sockaddr *addr)
 	return rtn;
 }
 
-cidr *server::getPolicy(struct sockaddr *addr)
+stack::subnet *server::getPolicy(struct sockaddr *addr)
 {
 	assert(addr != NULL);
 	assert(cfg != NULL);
 
-	cidr *policy;
+	stack::subnet *policy;
 
 	if(!cfg)
 		return NULL;
 
 	locking.access();
-	policy = cidr::find(((server *)(cfg))->acl, addr);
+	policy = (stack::subnet *)cidr::find(((server *)(cfg))->acl, addr);
 	if(!policy)
 		locking.release();
 	return policy;
