@@ -52,6 +52,54 @@ static bool tobool(const char *s)
 
 stack stack::sip;
 
+stack::subnet::subnet(cidr::policy **acl, const char *addr, const char *id) :
+cidr(acl, addr, id)
+{
+	struct sockaddr_storage dest;
+	struct sockaddr_in *sin = (struct sockaddr_in *)&dest;
+#ifdef	AF_INET6
+	struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)&dest;
+#endif
+	unsigned char *lp;
+	unsigned bits = getMask();
+
+	String::set(netname, sizeof(netname), id);
+	memset(&dest, 0, sizeof(dest));
+	sin->sin_family = family;
+	memcpy(&sin->sin_addr, &network, sizeof(network));
+	switch(family) {
+	case AF_INET:
+		sin->sin_port = htons(1);
+		lp = (unsigned char *)(&sin->sin_addr) + sizeof(sin->sin_addr) - 1;
+		if(bits < 31)
+			++*lp;
+		break;
+#ifdef	AF_INET6
+	case AF_INET6:
+		sin6->sin6_port = htons(1);
+		lp = (unsigned char *)(&sin6->sin6_addr) + sizeof(sin6->sin6_addr) - 1;
+		if(bits < 127)
+			++*lp;
+		break;
+#endif
+	default:
+		return;
+	} 
+	if(Socket::getinterface((struct sockaddr *)&iface, (struct sockaddr *)&dest))
+		memset(&iface, 0, sizeof(iface));
+	// gateway special rule to specify a gateway public interface...
+	else if(String::equal(id, "gateway")) {
+		String::set(netname, sizeof(netname), "*");
+		char buf[256];
+		Socket::getaddress((struct sockaddr *)&iface, buf, sizeof(buf));
+		service::publish(buf);
+	}
+	else if(!isMember((struct sockaddr *)&iface)) {
+		String::set(netname, sizeof(netname), "*");
+		service::published(&iface);
+	}
+}
+
 stack::segment::segment(call *cr, int cid, int did, int tid) : OrderedObject()
 {
 	assert(cr != NULL);
@@ -376,6 +424,52 @@ int stack::getDialog(session *s)
 	return did;
 }
 
+void stack::refer(session *source, eXosip_event_t *sevent)
+{
+	assert(source);
+	assert(sevent);
+
+	osip_header_t *header = NULL;
+	osip_message_t *msg = NULL;
+	session *target = NULL;
+	call *cr = source->parent;
+	int did;
+
+	if(cr->source == source)
+		target = cr->target;
+	else if(cr->target == source)
+		target = cr->source;
+
+	osip_message_header_get_byname(sevent->request, "Refer-To", 0, &header);
+	if(!header || !header->hvalue)
+		goto norefer;
+
+	did = getDialog(target);
+	if(did < 1) {
+norefer:
+		eXosip_lock();
+		goto failed;
+	}
+
+	eXosip_lock();
+	eXosip_call_build_refer(did, header->hvalue, &msg);
+	if(!msg) {
+failed:
+		eXosip_call_build_answer(sevent->tid, SIP_SERVICE_UNAVAILABLE, &msg);
+		if(msg)
+			eXosip_call_send_answer(sevent->tid, SIP_SERVICE_UNAVAILABLE, msg);
+		eXosip_unlock();
+		return;
+	}
+	osip_message_set_header(msg, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
+	osip_message_set_header(msg, ALLOW_EVENTS, "talk, hold, refer");
+	osip_message_set_header(msg, "Referred-By", source->identity);
+	eXosip_call_send_request(did, msg);
+	target->state = session::REFER;
+	target->tid = sevent->tid;
+	eXosip_unlock();
+}
+
 void stack::infomsg(session *source, eXosip_event_t *sevent)
 {
 	assert(source);
@@ -767,6 +861,8 @@ void stack::reload(service *cfg)
 				anon = value;
 			else if(!stricmp(key, "published") || !stricmp(key, "public"))
 				published = cfg->dup(value);
+			else if(!stricmp(key, "peering") || !stricmp(key, "gateway"))
+				service::publish(value);
 			else if(!stricmp(key, "proxy") || !stricmp(key, "outbound"))
 				new_proxy = cfg->dup(value);
 			else if(!stricmp(key, "agent") && !isConfigured())
@@ -784,7 +880,7 @@ void stack::reload(service *cfg)
 		}
 		sp.next();
 	}
-
+	
 	while(is(tp)) {
 		key = tp->getId();
 		value = tp->getPointer();
@@ -1055,7 +1151,7 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 	assert(s != NULL && s->parent != NULL);
 	assert(uri_target != NULL);
 
-	rtpproxy::session proxyinfo;
+	struct sockaddr *target;
 	Socket::address resolve;
 	stack::session *invited;
 	stack::call *call = s->parent;
@@ -1081,8 +1177,6 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 	
 	snprintf(touri, sizeof(touri), "<%s>", uri_target);
 
-	proxyinfo.clear();
-	server::classify(&proxyinfo, &call->source->proxy, NULL);
 	invite = NULL;
 
 	eXosip_lock();
@@ -1159,15 +1253,24 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 	eXosip_unlock();
 	invited = stack::create(call, cid);
 	registry::incUse(NULL, stats::OUTGOING);
-	rtpproxy::copy(&invited->proxy, &proxyinfo);
 	uri::userid(uri_target, username, sizeof(username));
 	uri::hostid(uri_target, route, sizeof(route));
 	String::set(invited->identity, sizeof(invited->identity), uri_target);
 	String::set(invited->display, sizeof(invited->display), username);
 	snprintf(invited->from, sizeof(invited->from), "<%s>", uri_target);
 	resolve.set(route, 5060);
-	if(resolve.getAddr())
-		uri::identity(resolve.getAddr(), invited->sysident, username, sizeof(invited->sysident));
+	service::published(&invited->peering);
+	String::set(invited->network, sizeof(invited->network), "*");
+	target = resolve.getAddr();
+	if(target) {
+		uri::identity(target, invited->sysident, username, sizeof(invited->sysident));
+		stack::subnet *subnet = server::getPolicy(target);
+		if(subnet) {
+			invited->peering = subnet->iface;
+			String::set(invited->network, sizeof(invited->network), subnet->netname);
+		}
+		server::release(subnet);
+	}
 	else
 		snprintf(invited->sysident, sizeof(invited->sysident), "%s@unknown", username);
 
@@ -1260,7 +1363,6 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 	assert(s != NULL && s->parent != NULL);
 	assert(rr != NULL);
 
-	rtpproxy::session proxyinfo;
 	linked_pointer<registry::target> tp = rr->internal.targets;
 	stack::session *invited;
 	stack::call *call = s->parent;
@@ -1301,12 +1403,10 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 			goto next;
 		}
 
-		proxyinfo.clear();
-
 		// if proxy required, but not available, then we must skip this
 		// invite...
-		if(server::classify(&proxyinfo, &call->source->proxy, (struct sockaddr *)&tp->address) && !assign(call, 4))
-			goto next;
+		// if(server::classify(&proxyinfo, &call->source->proxy, (struct sockaddr *)&tp->address) && !assign(call, 4))
+		//	goto next;
 
 		invite = NULL;
 		eXosip_lock();
@@ -1360,7 +1460,7 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 		cid = eXosip_call_send_initial_invite(invite);
 		if(cid > 0) {
 			snprintf(seqid, sizeof(seqid), "%08x-%d", s->sequence, s->cid);
-			stack::sipAddress(&tp->iface, route, seqid, sizeof(route));
+			stack::sipAddress((struct sockaddr_internet *)&tp->peering, route, seqid, sizeof(route));
 			eXosip_call_set_reference(cid, route);
 			++count;
 		}
@@ -1373,7 +1473,9 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 		eXosip_unlock();
 
 		invited = stack::create(call, cid);
-		rtpproxy::copy(&invited->proxy, &proxyinfo);
+
+		String::set(invited->network, sizeof(invited->network), tp->network);
+		invited->peering = tp->peering;
 		
 		if(rr->ext) 
 			snprintf(invited->sysident, sizeof(invited->sysident), "%u", rr->ext);
@@ -1383,7 +1485,7 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 			String::set(invited->display, sizeof(invited->display), rr->display);
 		else
 			String::set(invited->display, sizeof(invited->display), invited->sysident);
-		stack::sipPublish(&tp->iface, invited->identity, invited->sysident, sizeof(invited->identity));
+		stack::sipPublish((struct sockaddr_internet *)&tp->peering, invited->identity, invited->sysident, sizeof(invited->identity));
 		if(rr->ext && !rr->display[0])
 			snprintf(invited->from, sizeof(invited->from), 
 				"\"%s\" <%s;user=phone>", invited->sysident, invited->identity);
