@@ -268,20 +268,32 @@ void thread::invite(void)
 	if(!domain)
 		domain = requesting;
 
+	if(!access)
+		getsource();
+
+	if(String::equal(network, "decline")) {
+		send_reply(SIP_DECLINE);
+		call->failed(this, session);
+		return;
+	}
+
+	if(String::equal(network, "reject")) {
+		send_reply(SIP_SERVICE_UNAVAILABLE);
+		call->failed(this, session);
+		return;
+	}
+
 	// FIXME: we should get proxy count extimate from sdp into global thread object...
 
 	// assign initial proxy if required to accept call...
 	// if no proxy available, then return 503...
-	if(server::classify(&session->proxy, &call->source->proxy, via_address.getAddr())) {
-		if(!stack::assign(call, 4)) {
-noproxy:
-			send_reply(SIP_SERVICE_UNAVAILABLE);
-			call->failed(this, session);
-			return;
-		}
-	}
-	else if((destination == REDIRECTED || destination == EXTERNAL) && server::isProxied() && !stack::assign(call, 4))
-		goto noproxy;
+//	if(server::classify(&session->proxy, &call->source->proxy, via_address.getAddr())) {
+//		if(!stack::assign(call, 4)) 
+//			goto noproxy;
+//	}
+//	else 
+//	if((destination == REDIRECTED || destination == EXTERNAL) && server::isProxied() && !stack::assign(call, 4))
+//		goto noproxy;
 		
 	if(extension)
 		snprintf(fromext, sizeof(fromext), "%u", extension);
@@ -329,6 +341,7 @@ noproxy:
 	cdrnode->sequence = session->sequence;
 	cdrnode->cid = session->cid;
 	String::set(cdrnode->uuid, sizeof(cdrnode->uuid), session->uuid);	
+	String::set(cdrnode->network, sizeof(cdrnode->network), network);
 	
 	call->type = destination;
 	switch(destination) {
@@ -519,6 +532,12 @@ exit:
 
 	call->trying(this);
 	debug(2, "call proceeding %08x:%u\n", session->sequence, session->cid);
+	return;
+
+noproxy:
+	send_reply(SIP_SERVICE_UNAVAILABLE);
+	call->failed(this, session);
+	return;
 }
 
 void thread::identify(void)
@@ -1267,6 +1286,18 @@ bool thread::getsource(void)
 		
 	via_address.set(via_host, via_port);
 	access = server::getPolicy(via_address.getAddr());
+	if(access) {
+		peering = access->iface;
+		String::set(network, sizeof(network), access->netname);
+	}
+	else {
+		service::published(&peering);
+		String::set(network, sizeof(network), "*");
+	}
+	if(session) {
+		session->peering = peering;
+		String::set(session->network, sizeof(session->network), network);
+	}
 	return true;
 }
 
@@ -1510,9 +1541,9 @@ void thread::reregister(const char *contact, time_t interval)
 	refresh = reginfo->refresh(via_address, expire, contact);
 	if(!refresh) {
 		if(reginfo->type == MappedRegistry::USER && (reginfo->profile.features & USER_PROFILE_MULTITARGET))
-			count = reginfo->addTarget(via_address, expire, contact);
+			count = reginfo->addTarget(via_address, expire, contact, network, (struct sockaddr *)&peering);
 		else
-			count = reginfo->setTarget(via_address, expire, contact);
+			count = reginfo->setTarget(via_address, expire, contact, network, (struct sockaddr *)&peering);
 	}
 	if(refresh) 
 		debug(2, "refreshing %s for %ld seconds from %s:%u", getIdent(), interval, via_host, via_port);
@@ -1768,8 +1799,11 @@ void thread::run(void)
 			if(!session)
 				break;
 
-			if(session->state == stack::session::REINVITE) {
+			switch(session->state) {
+			case stack::session::REINVITE:
+			case stack::session::REFER:
 				session->parent->relay(this, session);
+			default:
 				break;
 			}
 			// copy target sdp into session object...
@@ -1860,6 +1894,7 @@ void thread::run(void)
 				}
 			}
 			break;
+
 		case EXOSIP_CALL_REINVITE:
 			stack::siplog(sevent->request);
 			authorizing = CALL;
@@ -1932,6 +1967,12 @@ void thread::run(void)
 				else
 					send_reply(SIP_NOT_FOUND);
 			}
+			else if(MSG_IS_REFER(sevent->request)) {
+				if(sevent->cid > 0)
+					session = stack::access(sevent->cid);
+				if(session) 
+					stack::refer(session, sevent);
+			}
 			break;
 		case EXOSIP_MESSAGE_NEW:
 			stack::siplog(sevent->request);
@@ -1945,6 +1986,12 @@ void thread::run(void)
 				authorizing = REGISTRAR;
 				registration();
 			}
+			else if(MSG_IS_REFER(sevent->request)) {
+				if(sevent->cid > 0)
+					session = stack::access(sevent->cid);
+				if(session) 
+					stack::refer(session, sevent);
+			}
 			else if(MSG_IS_BYE(sevent->request)) {
 				if(sevent->cid > 0)
 					session = stack::access(sevent->cid);
